@@ -65,6 +65,10 @@ sub protocol { $ENV{SERVER_PROTOCOL} || 'HTTP/1.0' }
 
 sub hostname { $_[0]->{query}->server_name }
 
+# CGI says "use this when using virtual hosts".  It falls back to
+# CGI->server_port.
+sub get_server_port { $_[0]->{query}->virtual_port }
+
 # Fake it by just giving the current time.
 sub request_time { time }
 
@@ -162,6 +166,8 @@ sub http_header {
 sub send_http_header {
     my $self = shift;
 
+    return if $self->http_header_sent;
+
     print STDOUT $self->http_header;
 
     $self->{http_header_sent} = 1;
@@ -283,7 +289,7 @@ sub send_fd {
 # Do something with ErrorDocument?
 # sub custom_response {}
 
-# I think we'ev made this essentially the same thing.
+# I think we've made this essentially the same thing.
 BEGIN {
     local $^W;
     *send_cgi_header = \&send_http_header;
@@ -452,7 +458,11 @@ sub NEXTKEY {
 
 sub cgi_headers {
     my $self = shift;
-    map { $_ => $self->{$_}[1] } keys %$self;
+    map { _map_header_key_to_cgi_key($_) => $self->{$_}[1] } keys %$self;
+}
+
+sub _map_header_key_to_cgi_key {
+    return $_[0] eq '-set-cookie' ? '-cookies' : $_[0];
 }
 
 1;
